/**************************************************************************** 
*  Copyright 2008 - 2012 C&H Technologies, Inc.  All Rights Reserved.					    * 
*****************************************************************************/
/****************************************************************************
*
*	m214_ansi.h
*
*	Header file for the m214 ANSI-C source module. 
*
*	C&H Part Number: 11029269
*
*****************************************************************************/

#ifndef	M214_HEADER
#define	M214_HEADER

#ifndef PLATFORM
#define	PLATFORM	chwin
#endif

#include "apis.h"

/**************************************************************************** 
*  TypeDef's
*****************************************************************************/
typedef char Int8;
typedef unsigned char UInt8;
typedef unsigned short UInt16;
typedef short Int16;
typedef int Int32;
typedef unsigned int UInt32;
typedef unsigned short Bool;
typedef double Real64;

typedef struct
{ APIS_HANDLE apis_handle;
	UInt32				apis_error;
	Real64 			fw_rev;
	Real64 			hw_rev;
} M214_INSTR, *M214_HANDLE;


/**************************************************************************** 
*  Error Codes
*****************************************************************************/
#define		M214_SUCCESS															0

#define		M214_SPECIFIC_ERROR_BASE									0xBFFC0900
#define		M214_ERROR_CALVAL_ACCESS_TIMEOUT					(M214_SPECIFIC_ERROR_BASE + 0)
#define		M214_ERROR_PARAMETER1											(M214_SPECIFIC_ERROR_BASE + 1)
#define		M214_ERROR_PARAMETER2											(M214_SPECIFIC_ERROR_BASE + 2)
#define		M214_ERROR_PARAMETER3											(M214_SPECIFIC_ERROR_BASE + 3)
#define		M214_ERROR_PARAMETER4											(M214_SPECIFIC_ERROR_BASE + 4)
#define		M214_ERROR_PARAMETER5											(M214_SPECIFIC_ERROR_BASE + 5)
#define		M214_ERROR_PARAMETER6											(M214_SPECIFIC_ERROR_BASE + 6)
#define		M214_ERROR_PARAMETER7											(M214_SPECIFIC_ERROR_BASE + 7)
#define		M214_ERROR_PARAMETER8											(M214_SPECIFIC_ERROR_BASE + 8)
#define		M214_ERROR_PARAMETER9											(M214_SPECIFIC_ERROR_BASE + 9)
#define		M214_ERROR_PARAMETER10										(M214_SPECIFIC_ERROR_BASE + 10)
#define		M214_ERROR_FAIL_ID_QUERY									(M214_SPECIFIC_ERROR_BASE + 11)
#define		M214_ERROR_APIS														(M214_SPECIFIC_ERROR_BASE + 12)
#define		M214_ERROR_OUT_OF_MEMORY                  (M214_SPECIFIC_ERROR_BASE + 13)
#define		M214_SPECIFIC_ERROR_END										(M214_SPECIFIC_ERROR_BASE + 14)
#define		M214_SPECIFIC_ERROR_ABSOLUTE_MAX					(M214_SPECIFIC_ERROR_BASE + 0x6FFF)


/**************************************************************************** 
*  Warning Codes
*****************************************************************************/
#define		M214_SPECIFIC_WARNING_BASE								0x3FFC0900
#define   M214_SPECIFIC_WARNING_END									(M214_SPECIFIC_WARNING_BASE + 1)
#define		M214_SPECIFIC_WARNING_ABSOLUTE_MAX				(M214_SPECIFIC_WARNING_BASE + 0x6FF)


/**************************************************************************** 
*  M-Module ID Words
*****************************************************************************/
#define		M214_IDENT_WORD_SYNC_CODE					0
#define		M214_IDENT_WORD_MODULE_NUM				1
#define		M214_IDENT_WORD_REVISION_NUM 			2
#define		M214_IDENT_WORD_MODULE_CHAR 			3
#define		M214_IDENT_WORD_VXI_SYNC_CODE 		16
#define		M214_IDENT_WORD_VXI_ID 						17
#define		M214_IDENT_WORD_VXI_DEVICE_TYPE  	18


/**************************************************************************** 
*	M214 Register offsets
*****************************************************************************/
#define		M214_ID_REG									      0x00			
#define		M214_REVISION_REG						      0x02
#define		M214_TEMPERATURE_REG				      0x04
#define		M214_ANALOG_INPUT_REG				      0x06
#define		M214_CALIBRATION_ADR_REG		      0x08
#define		M214_CALIBRATION_DATA_REG		      0x0A
#define		M214_VOLT_CTRL_STAT_REG						0x0C		// only valid for fw_rev < 2.0
#define		M214_RESERVED_0x0E								0x0E
#define   M214_RESERVED_0x0F								0x0F
#define		M214_VOLT_SRC0_MSB_REG						0x10
#define		M214_VOLT_SRC0_LSB_REG						0x12
#define		M214_VOLT_SRC1_MSB_REG						0x14
#define		M214_VOLT_SRC1_LSB_REG						0x16
#define		M214_VOLT_SRC2_MSB_REG						0x18
#define		M214_VOLT_SRC2_LSB_REG						0x1A
#define		M214_VOLT_SRC3_MSB_REG						0x1C
#define		M214_VOLT_SRC3_LSB_REG						0x1E
#define		M214_ID_ACCESS_REGISTER     			0xFE


/**************************************************************************** 
*	Bit Definitions
*****************************************************************************/
/* ID Register */
#define		M214_MODEL												0x00FF
#define		M214_CONFIGURATION								0xFF00
#define		M214_CONFIG_BIT_NUM								8

/* Revision Register */
#define		M214_HWMIN												0x000F
#define		M214_HWMAJ												0x00F0
#define		M214_FWMIN												0x0F00
#define		M214_FWMAJ												0xF000
#define		M214_FW_REV_BIT_NUM								8

/* Temperature Register */
#define		M214_TEMP_1												0x00FF
#define		M214_TEMP_2												0xFF00

/* Ext Analog Input Register */
#define		M214_VOLTAGE											0x03FF

/* Calibration Address Register */
#define		M214_CAL_ADDRESS									0x03FF
#define		M214_CAL_READ_MASK								0x0800
#define		M214_CAL_WRITE_MASK								0x0400
#define		M214_CALIBRATION_BIT_SET					1
#define		M214_CALIBRATION_BIT_NOT_SET			0


#define		M214_SC0													0x1000
#define		M214_SC1													0x2000
#define		M214_SC2													0x4000
#define		M214_SC3													0x8000

/* Calibration Data Port Register */
#define		M214_CAL_DATA											0xFFFF


/* Voltage Src X High Register */
#define		M214_OUTPUT_LEVEL_MSB							0x000F
#define		M214_RANGE												0x0030
#define		M214_RANGE3												0x00C0 // used in fw_rev before 2.0
#define		M214_RANGE2												0x0030 // used in fw_rev before 2.0
#define		M214_RANGE1												0x000C // used in fw_rev before 2.0
#define		M214_RANGE0												0x0003 // used in fw_rev before 2.0
#define		M214_OE														0x0100
#define		M214_OUTPUT_ENABLE_0_BIT_NUM			8			// only valid for fw_rev < 2.0
#define		M214_OUTPUT_ENABLE_1_BIT_NUM			9			// only valid for fw_rev < 2.0
#define		M214_OUTPUT_ENABLE_2_BIT_NUM			10		// only valid for fw_rev < 2.0
#define		M214_OUTPUT_ENABLE_3_BIT_NUM			11		// only valid for fw_rev < 2.0
#define		M214_RANGE_BIT_NUM								4				// only valid for fw_rev >= 2.0
#define		M214_OUTPUT_ENABLE_BIT_NUM				8				// only valid for fw_rev >= 2.0
#define		M214_READY_BIT_NUM								14			// only valid for fw_rev >= 2.0
#define		M214_RDY													0x4000 // NOT present in fw_rev before 2.0
#define		M214_TC														0x8000

/* Voltage Src X Low Register */
#define		M214_OUTPUT_LEVEL_LSB							0xFFFF


/**************************************************************************** 
*	Constans & Coefficiants
*****************************************************************************/
#define   M214_CAL_WAIT_TIMEOUT_SEC      		 0.200
#define		M214_DEGREES_PER_BIT							500e-3   		// DEGREES CELCIUS (range starts at -20 degrees C)
#define		M214_TEMPERATURE_COEFF						2
#define		M214_TEMPERATURE_OFFSET						20
#define		M214_ANALOG_VOLT_COEFF						4.9e-3			// 4.9 mV / bit (range starts at 0 volts) 
#define		M214_NEG_mV_RANGE									-99.0e-3
#define		M214_NEG_2V_RANGE									-2.0
#define		M214_NEG_10V_RANGE								-10.0
#define		M214_POS_mV_RANGE									99.0e-3
#define		M214_POS_2V_RANGE									2.0
#define		M214_POS_10V_RANGE								10.0
//#define		M214_RANGE1_COEFF									0.188827694e-6		// volts per bit given range
#define		M214_RANGE1_FULLSCALE_uV					198000
//#define		M214_RANGE2_COEFF									3.814700904e-6
#define		M214_RANGE2_FULLSCALE_uV					4000000
//#define		M214_RANGE3_COEFF									19.07350452e-6
#define		M214_RANGE3_FULLSCALE_uV					20000000
#define		M214_RANGE_0											0x00    // disables DAC (always 0 volts)
#define		M214_RANGE_1											0x01
#define		M214_RANGE_2											0x10
#define		M214_RANGE_3											0x11 

/* Channels  */
#define		M214_CHANNEL_1										0
#define		M214_CHANNEL_2										1
#define		M214_CHANNEL_3										2
#define		M214_CHANNEL_4										3

/* Booleans */
#define		M214_TRUE													1
#define		M214_FALSE												2


/**************************************************************************** 
*	Exported Function Prototypes
*****************************************************************************/
/*  Temperature  */
Int32 m214_GetTemperature (M214_HANDLE handle, Real64 *temp1, Real64 *temp2);             // BS

/* Analog Input */
Int32 m214_GetAnalogInput (M214_HANDLE handle, Real64 *voltage);							            // BS

/* Calibration */
Int32 m214_WaitForCalibrationAccess (M214_HANDLE handle, UInt16 mask);          						                            // BS
Int32 m214_ReadCalibrationValue (M214_HANDLE handle, UInt16 address, UInt16 *data);				                // BS
Int32 m214_WriteCalibrationValue (M214_HANDLE handle, UInt16 address, UInt16 data);				                // BS
Int32 m214_QueryCalibrationWriteBit (M214_HANDLE handle, Bool *value);										                // BS
Int32 m214_QueryCalibrationReadBit (M214_HANDLE handle, Bool *value);											                // BS

/* Set/Get Voltages and Configure Outputs  */
Int32 m214_SetVoltage (M214_HANDLE handle, UInt8 channel, Real64 voltage);
Int32 m214_GetVoltage (M214_HANDLE handle, UInt8 channel, Real64 *voltage);
Int32 m214_SetOutputEnable (M214_HANDLE handle, UInt8 channel, Bool enable);
Int32 m214_GetOutputEnable (M214_HANDLE handle, UInt8 channel, Bool *enable);
Int32 m214_SetRange (M214_HANDLE handle, UInt8 channel, UInt8 range);
Int32 m214_GetRange (M214_HANDLE handle, UInt8 channel, UInt8 *range);

/*  Utility Functions  */
Int32 m214_RevisionQuery(M214_HANDLE handle, Int8 *driver_rev, UInt8 *fw_rev, UInt8 *hw_rev);
Int32 m214_ErrorMessage (Int32 statusCode, Int8 errorMessage[]);
Int32 m214_GetIdentWord (M214_HANDLE handle, UInt16 offset, UInt16 *value);                               
Int32 m214_ReadID (M214_HANDLE handle, UInt8 *configuration, UInt8 *model);               // BS

/*****************************************************************************
*	Modify the functions below this point to use the M214 driver in an 
*	environment not supported by an APIS platform support module. All functions
*	above this point should not have to be modified.
*
*	In general, any call to an APIS function (apis_func()) will need to be
*	modified to communicate with the instrument or operating system being used.
*
******************************************************************************/
Int32 m214_Init (Int32 instr, Bool IDQuery, Bool reset, M214_HANDLE *handle);
Int32 m214_GetApisError (M214_HANDLE handle, UInt32 *apis_error);
Int32 m214_Close (M214_HANDLE handle);                      
Int32 m214_Reset (M214_HANDLE handle);
Int32 m214_ReadRegister (M214_HANDLE handle, UInt16 offset, UInt16 *value);
Int32 m214_WriteRegister (M214_HANDLE handle, UInt16 offset, UInt16 value);


#endif   //M214_HEADER
